use std::{
    io,
    path::{Path, PathBuf},
    str::FromStr,
};

use derive_more::derive::Debug;
use ron::de::SpannedError;
use serde::{Deserialize, Serialize};

use crate::cli::{GivenDay, GivenTask};
use thiserror::Error;

#[derive(Debug, Serialize, Deserialize, Clone, PartialEq, Eq)]
pub struct FileToBenchmark {
    given_day: GivenDay,
    given_task: GivenTask,
    where_to_look: PathBuf,
    expected_output: String,
}

impl From<FileToBenchmark> for (GivenDay, GivenTask, String) {
    fn from(val: FileToBenchmark) -> Self {
        (val.given_day, val.given_task, val.expected_output)
    }
}

impl FromStr for FileToBenchmark {
    type Err = CouldNotReadFileToBenchmark;

    fn from_str(s: &str) -> Result<Self, Self::Err> {
        let path = PathBuf::from(s);
        let content = std::fs::read_to_string(&path).map_err(|error| {
            if error.kind() == io::ErrorKind::NotFound {
                CouldNotReadFileToBenchmark::NotFound(s.to_owned())
            } else {
                error.into()
            }
        })?;
        let parsed_content = ron::from_str(&content)?;
        Ok(parsed_content)
    }
}

#[derive(Debug, Error)]
pub enum CouldNotReadFileToBenchmark {
    #[error("There is not file at {0:?}")]
    NotFound(String),
    #[error("Coud not read file at.\nDetails: {0}")]
    IoError(#[from] io::Error),
    #[error("File content is not in the correct format. Details: {0}")]
    InvalidRon(#[from] SpannedError),
}

impl FileToBenchmark {
    pub fn given_day(&self) -> GivenDay {
        self.given_day
    }

    pub fn given_task(&self) -> GivenTask {
        self.given_task
    }

    pub fn where_to_look(&self) -> &Path {
        &self.where_to_look
    }

    pub fn expected_output(&self) -> &str {
        &self.expected_output
    }
}
